@@ -1,39 +1,43 @@
 # -*- coding: utf8 -*-
 
-from nr.databind.core import Field, Struct
-from nr.interface import implements, override
-from pathlib import Path
-from pydoc_markdown.interfaces import Renderer
-from pydoc_markdown.contrib.renderers.markdown import MarkdownRenderer
-from typing import Any, Dict, List, Text, Tuple, Union
-import docspec
+import dataclasses
 import json
 import logging
-import os.path
+import os
+import typing as t
+import typing_extensions as te
+from pathlib import Path
+
+import databind.core.annotations as A
+import docspec
+
+from pydoc_markdown.interfaces import Renderer
+from pydoc_markdown.contrib.renderers.markdown import MarkdownRenderer
 
 logger = logging.getLogger(__name__)
 
 
+@dataclasses.dataclass
 class CustomizedMarkdownRenderer(MarkdownRenderer):
   """We override some defaults in this subclass. """
 
   #: Disabled because Docusaurus supports this automatically.
-  insert_header_anchors = Field(bool, default=False)
+  insert_header_anchors: bool = False
 
   #: Escape html in docstring, otherwise it could lead to invalid html.
-  escape_html_in_docstring = Field(bool, default=True)
+  escape_html_in_docstring: bool = True
 
   #: Conforms to Docusaurus header format.
-  render_module_header_template = Field(str, default=(
+  render_module_header_template: str = (
     '---\n'
     'sidebar_label: {relative_module_name}\n'
     'title: {module_name}\n'
     '---\n\n'
-  ))
+  )
 
 
-@implements(Renderer)
-class DocusaurusRenderer(Struct):
+@dataclasses.dataclass
+class DocusaurusRenderer(Renderer):
   """
   Produces Markdown files and a `sidebar.json` file for use in a [Docusaurus v2][1] websites.
   It creates files in a fixed layout that reflects the structure of the documented packages.
@@ -48,30 +52,30 @@ class DocusaurusRenderer(Struct):
   """
 
   #: The #MarkdownRenderer configuration.
-  markdown = Field(CustomizedMarkdownRenderer, default=CustomizedMarkdownRenderer)
+  markdown: te.Annotated[MarkdownRenderer, A.typeinfo(deserialize_as=CustomizedMarkdownRenderer)] = \
+    dataclasses.field(default_factory=CustomizedMarkdownRenderer)
 
   #: The path where the docusaurus docs content is. Defaults "docs" folder.
-  docs_base_path = Field(str, default='docs')
+  docs_base_path: str = 'docs'
 
   #: The output path inside the docs_base_path folder, used to ouput the
   #: module reference.
-  relative_output_path = Field(str, default='reference')
+  relative_output_path: str = 'reference'
 
   #: The sidebar path inside the docs_base_path folder, used to ouput the
   #: sidebar for the module reference.
-  relative_sidebar_path = Field(str, default='sidebar.json')
+  relative_sidebar_path: str = 'sidebar.json'
 
   #: The top-level label in the sidebar. Default to 'Reference'. Can be set to null to
   #: remove the sidebar top-level all together. This option assumes that there is only one top-level module.
-  sidebar_top_level_label = Field(str, default='Reference', nullable=True)
+  sidebar_top_level_label: t.Optional[str] = 'Reference'
 
   #: The top-level module label in the sidebar. Default to null, meaning that the actual
   #: module name will be used. This option assumes that there is only one top-level module.
-  sidebar_top_level_module_label = Field(str, default=None, nullable=True)
+  sidebar_top_level_module_label: t.Optional[str] = None
 
-  @override
-  def render(self, modules: List[docspec.Module]) -> None:
-    module_tree: Dict[str, Any] = {"children": {}, "edges": []}
+  def render(self, modules: t.List[docspec.Module]) -> None:
+    module_tree: t.Dict[str, t.Any] = {"children": {}, "edges": []}
     output_path = Path(self.docs_base_path) / self.relative_output_path
     for module in modules:
       filepath = output_path
@@ -108,7 +112,7 @@ class DocusaurusRenderer(Struct):
 
     self._render_side_bar_config(module_tree)
 
-  def _render_side_bar_config(self, module_tree: Dict[Text, Any]) -> None:
+  def _render_side_bar_config(self, module_tree: t.Dict[t.Text, t.Any]) -> None:
     """
     Render sidebar configuration in a JSON file. See Docusaurus sidebar structure:
 
@@ -134,7 +138,7 @@ class DocusaurusRenderer(Struct):
       logger.info("Render file %s", sidebar_path)
       json.dump(sidebar, handle, indent=2, sort_keys=True)
 
-  def _build_sidebar_tree(self, sidebar: Dict[Text, Any], module_tree: Dict[Text, Any]) -> None:
+  def _build_sidebar_tree(self, sidebar: t.Dict[t.Text, t.Any], module_tree: t.Dict[t.Text, t.Any]) -> None:
     """
     Recursively build the sidebar tree, it follows Docusaurus sidebar structure:
 
@@ -152,7 +156,7 @@ class DocusaurusRenderer(Struct):
       self._build_sidebar_tree(child, child_tree)
       sidebar["items"].append(child)
 
-    def _sort_items(item: Union[Text, Dict[Text, Any]]) -> Tuple[int, Text]:
+    def _sort_items(item: t.Union[t.Text, t.Dict[t.Text, t.Any]]) -> t.Tuple[int, t.Text]:
       """Sort sidebar items. Order follows:
         1. modules containing items come first
         2. alphanumeric order is applied
